@@ -1,6 +1,7 @@
 
 using System;
 using System.Collections;
+using System.Diagnostics;
 using System.IO;
 using System.Xml;
 using Mono.Unix;
@@ -9,22 +10,239 @@ namespace Tomboy
 {
 	public delegate void NoteRenameHandler (Note sender, string old_title);
 
+	// Contains all pure note data, like the note title and note text.
+	public class NoteData
+	{
+		readonly string uri;
+		string title;
+		string text;
+		DateTime create_date;
+		DateTime change_date;
+
+		int cursor_pos;
+		int width, height;
+		int x, y;
+
+		const int noPosition = -1;
+
+		public NoteData (string uri)
+		{
+			this.uri = uri;
+			this.text = "";
+			x = noPosition;
+			y = noPosition;
+		}
+
+		public string Uri
+		{
+			get { return uri; }
+		}
+
+		public string Title
+		{
+			get { return title; }
+			set { title = value; }
+		}
+
+		public string Text
+		{
+			get { return text; }
+			set { text = value; }
+		}
+
+		public DateTime CreateDate
+		{
+			get { return create_date; }
+			set { create_date = value; }
+		}
+
+		public DateTime ChangeDate
+		{
+			get { return change_date; }
+			set { change_date = value; }
+		}
+
+		// FIXME: the next five attributes don't belong here (the data
+		// model), but belong into the view; for now they are kept here
+		// for backwards compatibility
+
+		public int CursorPosition
+		{
+			get { return cursor_pos; }
+			set { cursor_pos = value; }
+		}
+
+		public int Width
+		{
+			get { return width; }
+			set { width = value; }
+		}
+
+		public int Height
+		{
+			get { return height; }
+			set { height = value; }
+		}
+
+		public int X
+		{
+			get { return x; }
+			set { x = value; }
+		}
+
+		public int Y
+		{
+			get { return y; }
+			set { y = value; }
+		}
+
+		public void SetPositionExtent (int x, int y, int width, int height)
+		{
+			Debug.Assert (x >= 0 && y >= 0);
+			Debug.Assert (width > 0 && height > 0);
+
+			this.x = x;
+			this.y = y;
+			this.width = width;
+			this.height = height;
+		}
+
+		public bool HasPosition ()
+		{
+			return x != noPosition && y != noPosition;
+		}
+
+		public bool HasExtent ()
+		{
+			return width != 0 && height != 0;
+		}
+	}
+
+	// This class wraps a NoteData instance. Most method calls are
+	// forwarded to the wrapped instance, but there is special behaviour
+	// for the Text attribute. This class takes care that this attribute
+	// is synchronized with the contents of a NoteBuffer instance.
+	public class NoteDataBufferSynchronizer
+	{
+		readonly NoteData data;
+		NoteBuffer buffer;
+
+		public NoteDataBufferSynchronizer (NoteData data)
+		{
+			this.data = data;
+		}
+
+		public NoteData GetDataSynchronized ()
+		{
+			// Assert that Data.Text returns the current
+			// text from the text buffer.
+			SynchronizeText ();
+			return data;
+		}
+
+		public NoteData Data
+		{
+			get { return data; }
+		}
+
+		public NoteBuffer Buffer
+		{
+			get { return buffer; }
+			set
+			{
+				buffer = value;
+
+				// Don't create Undo actions during load
+				buffer.Undoer.FreezeUndo ();
+
+				// Load the stored xml text
+				if (!TextInvalid ()) {
+					NoteBufferArchiver.Deserialize (buffer, 
+									buffer.StartIter, 
+									data.Text);
+				}
+				buffer.Modified = false;
+
+				Gtk.TextIter cursor;
+				if (data.CursorPosition != 0) {
+					// Move cursor to last-saved position
+					cursor = buffer.GetIterAtOffset (data.CursorPosition);
+				} else {
+					// Avoid title line
+					cursor = buffer.GetIterAtLine (2);
+				}
+				buffer.PlaceCursor (cursor);
+
+				// New events should create Undo actions
+				buffer.Undoer.ThawUndo ();
+
+				buffer.Changed += BufferChanged;
+				buffer.TagApplied += BufferTagApplied;
+				buffer.TagRemoved += BufferTagRemoved;
+
+				InvalidateText ();
+			}
+		}
+
+		public string Text
+		{
+			get
+			{
+				SynchronizeText ();
+				return data.Text;
+			}
+			set { data.Text = value; }
+		}
+
+		// Custom Methods
+
+		void InvalidateText ()
+		{
+			data.Text = "";
+		}
+
+		bool TextInvalid ()
+		{
+			return data.Text == "";
+		}
+
+		void SynchronizeText ()
+		{
+			if (TextInvalid () && buffer != null) {
+				data.Text = NoteBufferArchiver.Serialize (buffer); 
+			}
+		}
+
+		// Callbacks
+
+		void BufferChanged (object sender, EventArgs args)
+		{
+			InvalidateText ();
+		}
+
+		void BufferTagApplied (object sender, Gtk.TagAppliedArgs args)
+		{
+			if (NoteTagTable.TagIsSerializable (args.Tag)) {
+				InvalidateText ();
+			}
+		}
+
+		void BufferTagRemoved (object sender, Gtk.TagRemovedArgs args)
+		{
+			if (NoteTagTable.TagIsSerializable (args.Tag)) {
+				InvalidateText ();
+			}
+		}
+	}
+
 	public class Note 
 	{
+		readonly NoteDataBufferSynchronizer data;
+
 		string filepath;
 
 		bool save_needed;
 
-		// Accessed by NoteArchiver...
-		internal string title;
-		internal string text;
-		internal int width, height;
-		internal int x, y;
-		internal int cursor_pos;
-		internal DateTime create_date;
-		internal DateTime change_date;
-		internal string version;
-
 		NoteManager manager;
 		NoteWindow window;
 		NoteBuffer buffer;
@@ -38,45 +256,39 @@ namespace Tomboy
 			Console.WriteLine (format, args);
 		}
 
-		Note ()
+		Note (NoteData data, string filepath, NoteManager manager)
 		{
+			this.data = new NoteDataBufferSynchronizer (data);
+			this.filepath = filepath;
+			this.manager = manager;
 			save_timeout = new InterruptableTimeout ();
 			save_timeout.Timeout += SaveTimeout;
-			x = -1;
-			y = -1;
 		}
 
-		// Create a new note stored in a file...
-		Note (string title, string filepath, NoteManager manager) 
-			: this ()
+		static string UrlFromPath (string filepath)
 		{
-			this.title = title;
-			this.filepath = filepath;
-			this.manager = manager;
-			this.create_date = DateTime.Now;
-			this.change_date = create_date;
+			return "note://tomboy/" +
+				Path.GetFileNameWithoutExtension (filepath);
 		}
 
-		// Internal constructor, used when loading from an exising filepath.
-		Note (string filepath, NoteManager manager)
-			: this ()
-		{
-			this.manager = manager;
-			this.filepath = filepath;
-			this.create_date = DateTime.MinValue;
-			this.change_date = File.GetLastWriteTime (filepath);
-		}
-
-		public static Note CreateNewNote (string title, 
-						  string filepath, 
+		public static Note CreateNewNote (string title,
+						  string filepath,
 						  NoteManager manager)
 		{
-			return new Note (title, filepath, manager);
+			NoteData data = new NoteData (UrlFromPath (filepath));
+			data.Title = title;
+			data.CreateDate = DateTime.Now;
+			data.ChangeDate = data.CreateDate;
+			return new Note (data, filepath, manager);
 		}
 
-		public static Note CreateExistingNote (string filepath, NoteManager manager)
+		public static Note CreateExistingNote (NoteData data,
+						       string filepath,
+						       NoteManager manager)
 		{
-			return new Note (filepath, manager);
+			data.CreateDate = DateTime.MinValue;
+			data.ChangeDate = File.GetLastWriteTime (filepath);
+			return new Note (data, filepath, manager);
 		}
 
 		public void Delete ()
@@ -92,15 +304,8 @@ namespace Tomboy
 		// Load from an existing Note...
 		public static Note Load (string read_file, NoteManager manager)
 		{
-			Note note = new Note (read_file, manager);
-			NoteArchiver.Read (read_file, note);
-
-			if (note.version != NoteArchiver.CURRENT_VERSION) {
-				// Note has old format, so rewrite it.  No need
-				// to reread, since we are not adding anything.
-				Logger.Log ("Updating note XML to newest format...");
-				NoteArchiver.Write (read_file, note);
-			}
+			NoteData data = NoteArchiver.Read (read_file, UrlFromPath (read_file));
+			Note note = CreateExistingNote (data, read_file, manager);
 
 			return note;
 		}
@@ -112,9 +317,9 @@ namespace Tomboy
 			if (!save_needed)
 				return;
 
-			Logger.Log ("Saving '{0}'...", title);
+			Logger.Log ("Saving '{0}'...", data.Data.Title);
 
-			NoteArchiver.Write (filepath, this);
+			NoteArchiver.Write (filepath, data.GetDataSynchronized ());
 		}
 
 		//
@@ -149,7 +354,7 @@ namespace Tomboy
 			if (args.Mark != buffer.InsertMark)
 				return;
 
-			cursor_pos = args.Location.Offset;
+			data.Data.CursorPosition = args.Location.Offset;
 
 			DebugSave ("BufferInsertSetMark queueing save");
 			QueueSave (false);
@@ -173,16 +378,13 @@ namespace Tomboy
 			window.GetPosition (out cur_x, out cur_y);
 			window.GetSize (out cur_width, out cur_height);
 
-			if (x == cur_x && 
-			    y == cur_y &&
-			    width == cur_width && 
-			    height == cur_height)
+			if (data.Data.X == cur_x && 
+			    data.Data.Y == cur_y &&
+			    data.Data.Width == cur_width && 
+			    data.Data.Height == cur_height)
 				return;
 
-			x = cur_x;
-			y = cur_y;
-			width = cur_width;
-			height = cur_height;
+			data.Data.SetPositionExtent (cur_x, cur_y, cur_width, cur_height);
 			
 			DebugSave ("WindowConfigureEvent queueing save");
 			QueueSave (false);
@@ -197,19 +399,17 @@ namespace Tomboy
 		// Set a 4 second timeout to execute the save.  Possibly
 		// invalidate the text, which causes a re-serialize when the
 		// timeout is called...
-		public void QueueSave (bool invalidate_text)
+		public void QueueSave (bool content_changed)
 		{
-			DebugSave ("Got QueueSave with invalidate = {0}", invalidate_text);
+			DebugSave ("Got QueueSave");
 
 			// Replace the existing save timeout.  Wait 4 seconds
 			// before saving...
 			save_timeout.Reset (4000);
 			save_needed = true;
 
-			// Force a re-get of text on save
-			if (invalidate_text) {
-				change_date = DateTime.Now;
-				text = null;
+			if (content_changed) {
+				data.Data.ChangeDate = DateTime.Now;
 			}
 		}
 
@@ -228,10 +428,7 @@ namespace Tomboy
 
 		public string Uri
 		{
-			get { 
-				return "note://tomboy/" + 
-					Path.GetFileNameWithoutExtension (filepath); 
-			}
+			get { return data.Data.Uri; }
 		}
 
 		public string FilePath 
@@ -242,14 +439,14 @@ namespace Tomboy
 
 		public string Title 
 		{
-			get { return title; }
+			get { return data.Data.Title; }
 			set {
-				if (title != value) {
+				if (data.Data.Title != value) {
 					if (window != null)
 						window.Title = value;
 
-					string old_title = title;
-					title = value;
+					string old_title = data.Data.Title;
+					data.Data.Title = value;
 
 					if (Renamed != null)
 						Renamed (this, old_title);
@@ -259,28 +456,8 @@ namespace Tomboy
 
 		public string XmlContent 
 		{
-			get {
-				if (text == null && buffer != null) {
-					DebugSave ("Re-serializing to XML...");
-					text = NoteBufferArchiver.Serialize (buffer); 
-				}
-				return text;
-			}
-			set {
-				if (buffer != null) {
-					buffer.Clear ();
-					buffer.Undoer.FreezeUndo ();
-					NoteBufferArchiver.Deserialize (buffer, 
-									buffer.StartIter, 
-									value);
-					buffer.Undoer.ThawUndo ();
-				}
-
-				text = value;
-
-				DebugSave ("Set of XmlContent queueing save");
-				QueueSave (false);
-			}
+			get { return data.Text; }
+			set { data.Text = value; }
 		}
 
 		public string TextContent
@@ -295,14 +472,19 @@ namespace Tomboy
 			}
 		}
 
+		public NoteData Data
+		{
+			get { return data.GetDataSynchronized (); }
+		}
+
 		public DateTime CreateDate 
 		{
-			get { return create_date; }
+			get { return data.Data.CreateDate; }
 		}
 
 		public DateTime ChangeDate 
 		{
-			get { return change_date; }
+			get { return data.Data.ChangeDate; }
 		}
 
 		public NoteManager Manager
@@ -337,32 +519,10 @@ namespace Tomboy
 		{
 			get {
 				if (buffer == null) {
-					Logger.Log ("Creating Buffer for '{0}'...", title);
+					Logger.Log ("Creating Buffer for '{0}'...", data.Data.Title);
 
 					buffer = new NoteBuffer (TagTable);
-
-					// Don't create Undo actions during load
-					buffer.Undoer.FreezeUndo ();
-
-					// Load the stored xml text
-					NoteBufferArchiver.Deserialize (buffer, 
-									buffer.StartIter, 
-									text);
-					buffer.Modified = false;
-
-					
-					Gtk.TextIter cursor;
-					if (cursor_pos != 0) {
-						// Move cursor to last-saved position
-						cursor = buffer.GetIterAtOffset (cursor_pos);
-					} else {
-						// Avoid title line
-						cursor = buffer.GetIterAtLine (2);
-					}
-					buffer.PlaceCursor (cursor);
-
-					// New events should create Undo actions
-					buffer.Undoer.ThawUndo ();
+					data.Buffer = buffer;
 
 					// Listen for further changed signals
 					buffer.Changed += BufferChanged;
@@ -382,11 +542,11 @@ namespace Tomboy
 					window.Destroyed += WindowDestroyed;
 					window.ConfigureEvent += WindowConfigureEvent;
 					
-					if (width != 0 && height != 0)
-						window.SetDefaultSize (width, height);
+					if (data.Data.HasExtent ())
+						window.SetDefaultSize (data.Data.Width, data.Data.Height);
 
-					if (x != -1 && y != -1)
-						window.Move (x, y);
+					if (data.Data.HasPosition ())
+						window.Move (data.Data.X, data.Data.Y);
 
 					// This is here because emiting inside
 					// OnRealized causes segfaults.
@@ -399,14 +559,14 @@ namespace Tomboy
 
 		public bool IsSpecial 
 		{
-			get { return title == Catalog.GetString ("Start Here"); }
+			get { return data.Data.Title == Catalog.GetString ("Start Here"); }
 		}
 
 		public bool IsNew 
 		{
 			get { 
 				// Note is new if created in the last 24 hours.
-				return create_date > DateTime.Now.AddHours (-24);
+				return data.Data.CreateDate > DateTime.Now.AddHours (-24);
 			}
 		}
 
@@ -457,13 +617,16 @@ namespace Tomboy
 			}
 		}
 
-		public static void Read (string read_file, Note note)
+		public static NoteData Read (string read_file, string uri)
 		{
-			Instance.ReadFile (read_file, note);
+			return Instance.ReadFile (read_file, uri);
 		}
 
-		public virtual void ReadFile (string read_file, Note note) 
+		public virtual NoteData ReadFile (string read_file, string uri) 
 		{
+			NoteData note = new NoteData (uri);
+			string version = "";
+
 			StreamReader reader = new StreamReader (read_file, 
 								System.Text.Encoding.UTF8);
 			XmlTextReader xml = new XmlTextReader (reader);
@@ -474,38 +637,38 @@ namespace Tomboy
 				case XmlNodeType.Element:
 					switch (xml.Name) {
 					case "note":
-						note.version = xml.GetAttribute ("version");
+						version = xml.GetAttribute ("version");
 						break;
 					case "title":
-						note.title = xml.ReadString ();
+						note.Title = xml.ReadString ();
 						break;
 					case "text":
 						// <text> is just a wrapper around <note-content>
 						// NOTE: Use .text here to avoid triggering a save.
-						note.text = xml.ReadInnerXml ();
+						note.Text = xml.ReadInnerXml ();
 						break;
 					case "last-change-date":
-						note.change_date = 
+						note.ChangeDate = 
 							XmlConvert.ToDateTime (xml.ReadString ());
 						break;
 					case "create-date":
-						note.create_date = 
+						note.CreateDate = 
 							XmlConvert.ToDateTime (xml.ReadString ());
 						break;
 					case "cursor-position":
-						note.cursor_pos = int.Parse (xml.ReadString ());
+						note.CursorPosition = int.Parse (xml.ReadString ());
 						break;
 					case "width":
-						note.width = int.Parse (xml.ReadString ());
+						note.Width = int.Parse (xml.ReadString ());
 						break;
 					case "height":
-						note.height = int.Parse (xml.ReadString ());
+						note.Height = int.Parse (xml.ReadString ());
 						break;
 					case "x":
-						note.x = int.Parse (xml.ReadString ());
+						note.X = int.Parse (xml.ReadString ());
 						break;
 					case "y":
-						note.y = int.Parse (xml.ReadString ());
+						note.Y = int.Parse (xml.ReadString ());
 						break;
 					}
 					break;
@@ -513,14 +676,23 @@ namespace Tomboy
 			}
 
 			xml.Close ();
+
+			if (version != NoteArchiver.CURRENT_VERSION) {
+				// Note has old format, so rewrite it.  No need
+				// to reread, since we are not adding anything.
+				Logger.Log ("Updating note XML to newest format...");
+				NoteArchiver.Write (read_file, note);
+			}
+
+			return note;
 		}
 
-		public static void Write (string write_file, Note note)
+		public static void Write (string write_file, NoteData note)
 		{
 			Instance.WriteFile (write_file, note);
 		}
 
-		public virtual void WriteFile (string write_file, Note note) 
+		public virtual void WriteFile (string write_file, NoteData note) 
 		{
 			string tmp_file = write_file + ".tmp";
 
@@ -545,26 +717,21 @@ namespace Tomboy
 				// Move the temp file to write_file
 				File.Move (tmp_file, write_file);
 			}
-
-			// This is always the latest after a write
-			note.version = CURRENT_VERSION;
 		}
 
-		public static void Write (TextWriter writer, Note note)
+		public static void Write (TextWriter writer, NoteData note)
 		{
 			Instance.WriteFile (writer, note);
 		}
 
-		public void WriteFile (TextWriter writer, Note note)
+		public void WriteFile (TextWriter writer, NoteData note)
 		{
 			XmlTextWriter xml = new XmlTextWriter (writer);
 			Write (xml, note);
 			xml.Close ();
-
-			note.version = CURRENT_VERSION;
 		}
 
-		void Write (XmlTextWriter xml, Note note)
+		void Write (XmlTextWriter xml, NoteData note)
 		{
 			xml.Formatting = Formatting.Indented;
 
@@ -584,45 +751,43 @@ namespace Tomboy
 						 "http://beatniksoftware.com/tomboy/size");
 
 			xml.WriteStartElement (null, "title", null);
-			xml.WriteString (note.title);
+			xml.WriteString (note.Title);
 			xml.WriteEndElement ();
 
 			xml.WriteStartElement (null, "text", null);
 			xml.WriteAttributeString ("xml", "space", null, "preserve");
 			// Insert <note-content> blob...
-			// NOTE: Use .XmlContent here to force a reget of text
-			//       from the buffer, if needed.
-			xml.WriteRaw (note.XmlContent);
+			xml.WriteRaw (note.Text);
 			xml.WriteEndElement ();
 
 			xml.WriteStartElement (null, "last-change-date", null);
-			xml.WriteString (XmlConvert.ToString (note.change_date));
+			xml.WriteString (XmlConvert.ToString (note.ChangeDate));
 			xml.WriteEndElement ();
 
-			if (note.create_date != DateTime.MinValue) {
+			if (note.CreateDate != DateTime.MinValue) {
 				xml.WriteStartElement (null, "create-date", null);
-				xml.WriteString (XmlConvert.ToString (note.create_date));
+				xml.WriteString (XmlConvert.ToString (note.CreateDate));
 				xml.WriteEndElement ();
 			}
 
 			xml.WriteStartElement (null, "cursor-position", null);
-			xml.WriteString (note.cursor_pos.ToString ());
+			xml.WriteString (note.CursorPosition.ToString ());
 			xml.WriteEndElement ();
 
 			xml.WriteStartElement (null, "width", null);
-			xml.WriteString (note.width.ToString ());
+			xml.WriteString (note.Width.ToString ());
 			xml.WriteEndElement ();
 
 			xml.WriteStartElement (null, "height", null);
-			xml.WriteString (note.height.ToString ());
+			xml.WriteString (note.Height.ToString ());
 			xml.WriteEndElement ();
 
 			xml.WriteStartElement (null, "x", null);
-			xml.WriteString (note.x.ToString ());
+			xml.WriteString (note.X.ToString ());
 			xml.WriteEndElement ();
 
 			xml.WriteStartElement (null, "y", null);
-			xml.WriteString (note.y.ToString ());
+			xml.WriteString (note.Y.ToString ());
 			xml.WriteEndElement ();
 
 			xml.WriteEndElement (); // Note
