@@ -2,7 +2,7 @@
 using System;
 using System.Runtime.InteropServices;
 using System.Text;
-using Mono.Posix;
+using Mono.Unix;
 
 namespace Tomboy
 {
@@ -56,8 +56,8 @@ namespace Tomboy
 		{
 			switch (args.Key) {
 			case Preferences.ENABLE_CUSTOM_FONT:
-				Console.WriteLine ("Switching note font {0}...", 
-						   (bool) args.Value ? "ON" : "OFF");
+				Logger.Log ("Switching note font {0}...", 
+					    (bool) args.Value ? "ON" : "OFF");
 
 				if ((bool) args.Value) {
 					string font_string = (string) 
@@ -69,8 +69,8 @@ namespace Tomboy
 				break;
 
 			case Preferences.CUSTOM_FONT_FACE:
-				Console.WriteLine ("Switching note font to '{0}'...", 
-						   (string) args.Value);
+				Logger.Log ("Switching note font to '{0}'...", 
+					    (string) args.Value);
 
 				ModifyFont (Pango.FontDescription.FromString ((string) args.Value));
 				break;
@@ -102,7 +102,7 @@ namespace Tomboy
 				StringBuilder insert = new StringBuilder ();
 
 				foreach (Uri uri in uri_list) {
-					Console.WriteLine ("Got Dropped URI: {0}", uri);
+					Logger.Log ("Got Dropped URI: {0}", uri);
 
 					// FIXME: The space here is a hack
 					// around a bug in the URL Regex which
@@ -117,10 +117,11 @@ namespace Tomboy
 				}
 
 				if (insert.Length > 0) {
-					Buffer.InsertWithTags (
-						Buffer.GetIterAtMark (Buffer.InsertMark),
-						insert.ToString (),
-						Buffer.TagTable.Lookup ("link:url"));
+					Gtk.TextIter insert_iter = 
+						Buffer.GetIterAtMark (Buffer.InsertMark);
+					Buffer.InsertWithTags (ref insert_iter,
+							       insert.ToString (),
+							       Buffer.TagTable.Lookup ("link:url"));
 				}
 
 				Gtk.Drag.Finish (context, insert.Length > 0, false, time);
@@ -145,13 +146,6 @@ namespace Tomboy
 		GlobalKeybinder global_keys;
 		InterruptableTimeout mark_set_timeout;
 
-		static Gdk.Pixbuf stock_notes;
-
-		static NoteWindow ()
-		{
-			stock_notes = new Gdk.Pixbuf (null, "stock_notes.png");
-		}
-
 		// 
 		// Construct a window to display a note
 		// 
@@ -163,7 +157,7 @@ namespace Tomboy
 			base (note.Title) 
 		{
 			this.note = note;
-			this.Icon = stock_notes;
+			this.IconName = "tomboy";
 			this.SetDefaultSize (450, 360);
 
 			accel_group = new Gtk.AccelGroup ();
@@ -304,36 +298,8 @@ namespace Tomboy
 
 		void DeleteButtonClicked () 
 		{
-			HIGMessageDialog dialog = 
-				new HIGMessageDialog (
-					note.Window,
-					Gtk.DialogFlags.DestroyWithParent,
-					Gtk.MessageType.Question,
-					Gtk.ButtonsType.None,
-					Catalog.GetString ("Really delete this note?"),
-					Catalog.GetString ("If you delete a note it is " +
-							   "permanently lost."));
-
-			Gtk.Button button;
-
-			button = new Gtk.Button (Gtk.Stock.Cancel);
-			button.CanDefault = true;
-			button.Show ();
-			dialog.AddActionWidget (button, Gtk.ResponseType.Cancel);
-			dialog.DefaultResponse = Gtk.ResponseType.Cancel;
-
-			button = new Gtk.Button (Gtk.Stock.Delete);
-			button.CanDefault = true;
-			button.Show ();
-			dialog.AddActionWidget (button, 666);
-
-			int result = dialog.Run ();
-			if (result == 666 ) {
-				// This will destroy our window...
-				note.Manager.Delete (note);
-			}
-
-			dialog.Destroy();
+			// Prompt for note deletion
+			NoteUtils.ShowDeletionDialog (note, this);
 		}
 
 		//
@@ -382,7 +348,7 @@ namespace Tomboy
 		{
 			args.Menu.AccelGroup = accel_group;
 
-			Console.WriteLine ("Populating context menu...");
+			Logger.Log ("Populating context menu...");
 
 			// Remove the lame-o gigantic Insert Unicode Control
 			// Characters menu item.
@@ -707,7 +673,6 @@ namespace Tomboy
 
 	public class NoteTextMenu : Gtk.Menu
 	{
-		Gtk.AccelGroup accel_group;
 		NoteBuffer buffer;
 		UndoManager undo_manager;
 		bool event_freeze;
@@ -744,7 +709,6 @@ namespace Tomboy
 				     UndoManager    undo_manager) 
 			: base ()
 		{
-			this.accel_group = accel_group;
 			this.buffer = buffer;
 			this.undo_manager = undo_manager;
 
@@ -977,7 +941,7 @@ namespace Tomboy
 		void UndoClicked (object sender, EventArgs args)
 		{
 			if (undo_manager.CanUndo) {
-				Console.WriteLine ("Running undo...");
+				Logger.Log ("Running undo...");
 				undo_manager.Undo ();
 			}
 		}
@@ -985,7 +949,7 @@ namespace Tomboy
 		void RedoClicked (object sender, EventArgs args)
 		{
 			if (undo_manager.CanRedo) {
-				Console.WriteLine ("Running redo...");
+				Logger.Log ("Running redo...");
 				undo_manager.Redo ();
 			}
 		}
