@@ -4,7 +4,7 @@ using System.Collections;
 using System.Diagnostics;
 using System.Reflection;
 using System.IO;
-using Mono.Posix;
+using Mono.Unix.Native;
 
 namespace Tomboy
 {
@@ -127,7 +127,9 @@ namespace Tomboy
 		static Type [] stock_plugins = 
 			new Type [] {
 				typeof (NoteRenameWatcher),
+#if ENABLE_GTKSPELL
 				typeof (NoteSpellChecker),
+#endif // ENABLE_GTKSPELL
 				typeof (NoteUrlWatcher),
 				typeof (NoteLinkWatcher),
 				typeof (NoteWikiWatcher),
@@ -148,19 +150,20 @@ namespace Tomboy
 				dir_watcher.Deleted += OnPluginDeleted;
 				dir_watcher.EnableRaisingEvents = true;
 			} catch (ArgumentException e) { 
-				Console.WriteLine ("Error creating a FileSystemWatcher on {0} : {1}",
-									plugins_dir, e);
+				Logger.Log ("Error creating a FileSystemWatcher on \"{0}\": {1}",
+					    plugins_dir, e.Message);
 				dir_watcher = null;
 			}
 			
 			try {
-				sys_dir_watcher = new FileSystemWatcher (Defines.SYS_PLUGINS_DIR, "*.dll");
+				sys_dir_watcher = 
+					new FileSystemWatcher (Defines.SYS_PLUGINS_DIR, "*.dll");
 				sys_dir_watcher.Created += OnPluginCreated;
 				sys_dir_watcher.Deleted += OnPluginDeleted;
 				sys_dir_watcher.EnableRaisingEvents = true;
 			} catch (ArgumentException e) {
-				Console.WriteLine ("Error creating a FileSystemWatcher on {0} : {1}", 
-									Defines.SYS_PLUGINS_DIR, e);
+				Logger.Log ("Error creating a FileSystemWatcher on \"{0}\": {1}", 
+					    Defines.SYS_PLUGINS_DIR, e.Message);
 				sys_dir_watcher = null;
 			}
 			
@@ -168,24 +171,35 @@ namespace Tomboy
 			plugin_hash = new Hashtable ();
 		}
 
+		// Run file manager for ~/.tomboy/Plugins
 		public void ShowPluginsDirectory ()
 		{
-			// Run file manager for ~/.tomboy/Plugins
-			// FIXME: There has to be a better way to check this...
+			string command, args;
 
+			// FIXME: There has to be a better way to check this...
 			if (Environment.GetEnvironmentVariable ("GNOME_DESKTOP_SESSION_ID") == null &&
 			    (Environment.GetEnvironmentVariable ("KDE_FULL_SESSION") != null ||
 			     Environment.GetEnvironmentVariable ("KDEHOME") != null ||
 			     Environment.GetEnvironmentVariable ("KDEDIR") != null)) {
-				Console.WriteLine ("Starting Konqueror...");
+				Logger.Log ("Starting Konqueror...");
 
-				Process.Start ("konqueror", plugins_dir);
+				command = "konqueror";
+				args = plugins_dir;
 			} else {
-				Console.WriteLine ("Starting Nautilus...");
+				Logger.Log ("Starting Nautilus...");
 
-				string args = string.Format ("--no-desktop --no-default-window {0}",
-							     plugins_dir);
-				Process.Start ("nautilus", args);
+				command = "nautilus";
+				args = string.Format ("--no-desktop --no-default-window {0}",
+						      plugins_dir);
+			}
+
+			try {
+				Process.Start (command, args);
+			} catch (SystemException e) {
+				Logger.Log ("Error opening file browser \"{0}\" to \"{1}\": {2}",
+					    command,
+					    plugins_dir,
+					    e.Message);
 			}
 		}
 
@@ -216,34 +230,34 @@ namespace Tomboy
 			// Remove old version 0.3.[12] "Uninstalled Plugins" symlink
 			string uninstalled_dir = Path.Combine (plugins_dir, "Uninstalled Plugins");
 			if (Directory.Exists (uninstalled_dir)) {
-				Console.WriteLine ("Removing old \"Uninstalled Plugins\" " +
+				Logger.Log ("Removing old \"Uninstalled Plugins\" " +
 						   "directory...");
 				try {
 					Syscall.unlink (uninstalled_dir);
 				} catch (Exception e) {
-					Console.WriteLine ("Error removing: {0}", e);
+					Logger.Log ("Error removing: {0}", e);
 				}
 			}
 
 			// Remove old version 0.3.[12] "ExportToHTML.dll" file
 			string export_to_html_dll = Path.Combine (plugins_dir, "ExportToHTML.dll");
 			if (File.Exists (export_to_html_dll)) {
-				Console.WriteLine ("Removing old \"ExportToHTML.dll\" plugin...");
+				Logger.Log ("Removing old \"ExportToHTML.dll\" plugin...");
 				try {
 					Syscall.unlink (export_to_html_dll);
 				} catch (Exception e) {
-					Console.WriteLine ("Error removing: {0}", e);
+					Logger.Log ("Error removing: {0}", e);
 				}
 			}
 
 			// Remove old version 0.3.[12] "PrintNotes.dll" file
 			string print_notes_dll = Path.Combine (plugins_dir, "PrintNotes.dll");
 			if (File.Exists (print_notes_dll)) {
-				Console.WriteLine ("Removing old \"PrintNotes.dll\" plugin...");
+				Logger.Log ("Removing old \"PrintNotes.dll\" plugin...");
 				try {
 					Syscall.unlink (print_notes_dll);
 				} catch (Exception e) {
-					Console.WriteLine ("Error removing: {0}", e);
+					Logger.Log ("Error removing: {0}", e);
 				}
 			}
 		}
@@ -264,7 +278,7 @@ namespace Tomboy
 				Assembly asm = Assembly.GetExecutingAssembly();
 				Stream stream = asm.GetManifestResourceStream (default_desktop);
 				if (stream != null) {
-					Console.WriteLine ("Writing '{0}'...", default_path);
+					Logger.Log ("Writing '{0}'...", default_path);
 					try {
 						StreamWriter file = File.CreateText (default_path);
 						StreamReader reader;
@@ -299,7 +313,7 @@ namespace Tomboy
 
 		void OnPluginCreated (object sender, FileSystemEventArgs args)
 		{
-			Console.WriteLine ("Plugin '{0}' Created", 
+			Logger.Log ("Plugin '{0}' Created", 
 					   Path.GetFileName (args.FullPath));
 
 			ArrayList asm_plugins = FindPluginTypesInFile (args.FullPath);
@@ -329,7 +343,7 @@ namespace Tomboy
 
 		void OnPluginDeleted (object sender, FileSystemEventArgs args)
 		{
-			Console.WriteLine ("Plugin '{0}' Deleted", 
+			Logger.Log ("Plugin '{0}' Deleted", 
 					   Path.GetFileName (args.FullPath));
 
 			ArrayList kill_list = new ArrayList ();
@@ -395,9 +409,9 @@ namespace Tomboy
 			try {
 				files = Directory.GetFiles (dirpath, "*.dll");
 			} catch (Exception e) {
-				Console.WriteLine ("Error getting plugin types from {0}: {1}", 
-						   dirpath, 
-						   e);
+				Logger.Log ("Error getting plugin types from {0}: {1}", 
+					    dirpath, 
+					    e.Message);
 				return dir_plugin_types;
 			}
 
@@ -410,7 +424,7 @@ namespace Tomboy
 						dir_plugin_types.Add (type);
 					}
 				} catch (Exception e) {
-					Console.WriteLine ("Failed.\n{0}", e);
+					Logger.Log ("Failed.\n{0}", e);
 				}
 			}
 
@@ -437,7 +451,7 @@ namespace Tomboy
 				}
 			}
 
-			Console.WriteLine ("{0}", found_one ? "Done." : "Skipping.");
+			Logger.Log ("{0}", found_one ? "Done." : "Skipping.");
 
 			return asm_plugins;
 		}
