@@ -1,7 +1,7 @@
 
 using System;
 using System.Runtime.InteropServices;
-using Mono.Posix;
+using Mono.Unix;
 
 using Gtk;
 
