@@ -1,6 +1,6 @@
 
 using System;
-using Mono.Posix;
+using Mono.Unix;
 using System.Text;
 
 namespace Tomboy
@@ -10,6 +10,7 @@ namespace Tomboy
 		NoteManager manager;
 
 		Gtk.AccelGroup accel_group;
+		Gtk.Label note_count;
 		Gtk.Button close_button;
 		Gtk.ScrolledWindow matches_window;
 		Gtk.VBox content_vbox;
@@ -17,21 +18,26 @@ namespace Tomboy
 		Gtk.TreeView tree;
 		Gtk.ListStore store;
 
-		static Gdk.Pixbuf recent_icon;
-		static Gdk.Pixbuf stock_notes;
+		static Type [] column_types = 
+			new Type [] {
+				typeof (Gdk.Pixbuf), // icon
+				typeof (string),     // title
+				typeof (string),     // change date
+				typeof (Note),       // note
+			};
+
+		static Gdk.Pixbuf note_icon;
 
 		static NoteRecentChanges ()
 		{
-			// FIXME: Get a better recent notes window icon
-			recent_icon = new Gdk.Pixbuf (null, "stock_notes.png");
-			stock_notes = GuiUtils.GetMiniIcon ("stock_notes.png");
+			note_icon = GuiUtils.GetIcon ("tomboy", 22);
 		}
 
 		public NoteRecentChanges (NoteManager manager)
 			: base (Catalog.GetString ("Table of Contents"))
 		{
 			this.manager = manager;
-			this.Icon = recent_icon;
+			this.IconName = "tomboy";
 			this.DefaultWidth = 200;
 
 			// For Escape (Close)
@@ -56,14 +62,17 @@ namespace Tomboy
 			MakeRecentTree ();
 			tree.Show ();
 
-			// List all the current notes
-			UpdateResults ();
+			note_count = new Gtk.Label ();
+			note_count.Show ();
 
 			// Update on changes to notes
 			manager.NoteDeleted += OnNotesChanged;
 			manager.NoteAdded += OnNotesChanged;
 			manager.NoteRenamed += OnNoteRenamed;
-			
+
+			// List all the current notes
+			UpdateResults ();
+
 			matches_window = new Gtk.ScrolledWindow ();
 			matches_window.ShadowType = Gtk.ShadowType.In;
 
@@ -94,8 +103,9 @@ namespace Tomboy
 			close_button.Show ();
 
 			Gtk.HButtonBox button_box = new Gtk.HButtonBox ();
-			button_box.Layout = Gtk.ButtonBoxStyle.End;
+			button_box.Layout = Gtk.ButtonBoxStyle.Edge;
 			button_box.Spacing = 8;
+			button_box.PackStart (note_count);
 			button_box.PackStart (close_button);
 			button_box.Show ();
 
@@ -111,14 +121,6 @@ namespace Tomboy
 
 		void MakeRecentTree ()
 		{
-			Type [] types = 
-				new Type [] {
-					typeof (Gdk.Pixbuf), // icon
-					typeof (string),     // title
-					typeof (string),     // change date
-					typeof (Note),       // note
-				};
-
 			Gtk.TargetEntry [] targets = 
 				new Gtk.TargetEntry [] {
 					new Gtk.TargetEntry ("STRING", 
@@ -132,18 +134,12 @@ namespace Tomboy
 							     1),
 				};
 
-			store = new Gtk.ListStore (types);
-			store.SetSortFunc (2 /* change date */,
-					   new Gtk.TreeIterCompareFunc (CompareDates),
-					   IntPtr.Zero, 
-					   null);
-			store.SetSortColumnId (2 /* change date */, Gtk.SortType.Descending);
-
-			tree = new Gtk.TreeView (store);
+			tree = new Gtk.TreeView ();
 			tree.HeadersVisible = true;
 			tree.RulesHint = true;
 			tree.RowActivated += OnRowActivated;
 			tree.DragDataGet += OnDragDataGet;
+			tree.KeyPressEvent += OnKeyPressed;
 
 			tree.EnableModelDragSource (Gdk.ModifierType.Button1Mask,
 						    targets,
@@ -152,7 +148,7 @@ namespace Tomboy
 			Gtk.CellRenderer renderer;
 
 			Gtk.TreeViewColumn title = new Gtk.TreeViewColumn ();
-			title.Title = "Note";
+			title.Title = Catalog.GetString ("Note");
 			title.Sizing = Gtk.TreeViewColumnSizing.Autosize;
 			title.Resizable = true;
 			
@@ -183,13 +179,39 @@ namespace Tomboy
 
 		void UpdateResults ()
 		{
-			store.Clear ();
-
 			// FIXME: Restore the currently highlighted note
 
-			foreach (Note note in manager.Notes) {
-				AppendResultTreeView (store, note);
+			int sort_column = 2; /* change date */
+			Gtk.SortType sort_type = Gtk.SortType.Descending;
+			if (store != null) {
+				store.GetSortColumnId (out sort_column, out sort_type);
 			}
+
+			store = new Gtk.ListStore (column_types);
+			store.SetSortFunc (2 /* change date */,
+					   new Gtk.TreeIterCompareFunc (CompareDates));
+
+			int cnt = 0;
+			foreach (Note note in manager.Notes) {
+				string nice_date = PrettyPrintDate (note.ChangeDate);
+
+				store.AppendValues (note_icon,  /* icon */
+						    note.Title, /* title */
+						    nice_date,  /* change date */
+						    note);      /* note */
+				cnt++;
+			}
+
+			// Set the sort column after loading data, since we
+			// don't want to resort on every append.
+			store.SetSortColumnId (sort_column, sort_type);
+
+			tree.Model = store;
+
+			note_count.Text = string.Format (Catalog.GetPluralString("Total: {0} note",
+										 "Total: {0} notes",
+										 cnt),
+							 cnt);
 		}
 
 		void OnNotesChanged (object sender, Note changed)
@@ -204,13 +226,7 @@ namespace Tomboy
 
 		void OnDragDataGet (object sender, Gtk.DragDataGetArgs args)
 		{
-			Gtk.TreeModel model;
-			Gtk.TreeIter iter;
-
-			if (!tree.Selection.GetSelected (out model, out iter))
-				return;
-
-			Note note = (Note) model.GetValue (iter, 3 /* note */);
+			Note note = GetSelectedNote ();
 			if (note == null)
 				return;
 
@@ -224,6 +240,17 @@ namespace Tomboy
 			args.SelectionData.Text = note.Title;
 		}
 
+		Note GetSelectedNote ()
+		{
+			Gtk.TreeModel model;
+			Gtk.TreeIter iter;
+
+			if (!tree.Selection.GetSelected (out model, out iter))
+				return null;
+
+			return (Note) model.GetValue (iter, 3 /* note */);
+		}
+
 		string PrettyPrintDate (DateTime date)
 		{
 			DateTime now = DateTime.Now;
@@ -246,17 +273,6 @@ namespace Tomboy
 				return date.ToString (Catalog.GetString ("MMMM d yyyy, h:mm tt"));
 		}
 
-		void AppendResultTreeView (Gtk.ListStore store, Note note)
-		{
-			string nice_date = PrettyPrintDate (note.ChangeDate);
-
-			Gtk.TreeIter iter = store.Append ();
-			store.SetValue (iter, 0 /* icon */, stock_notes);
-			store.SetValue (iter, 1 /* title */, note.Title);
-			store.SetValue (iter, 2 /* change date */, nice_date);
-			store.SetValue (iter, 3 /* note */, note);
-		}
-
 		void CloseClicked (object sender, EventArgs args)
 		{
 			Hide ();
@@ -284,5 +300,21 @@ namespace Tomboy
 
 			note.Window.Present ();
 		}
+
+		void OnKeyPressed (object obj, Gtk.KeyPressEventArgs args)
+		{
+			Gdk.EventKey eventKey = args.Event;
+
+			if (eventKey.Key == Gdk.Key.Delete && 
+			    eventKey.State == 0 /* Gdk.ModifierType.None */) {
+				// Get the selected note and prompt for deletion
+				Note note = GetSelectedNote();
+				if (note == null)
+					return;
+
+				NoteUtils.ShowDeletionDialog (note, this);
+			}		
+		}
+
 	}
 }
