@@ -3,7 +3,7 @@ using System;
 using System.Collections;
 using System.Runtime.InteropServices;
 using System.Text;
-using Mono.Posix;
+using Mono.Unix;
 
 namespace Tomboy 
 {
@@ -12,7 +12,7 @@ namespace Tomboy
 		NoteManager manager;
 
 		Gtk.AccelGroup accel_group;
-		Gtk.Combo find_combo;
+		Gtk.ComboBoxEntry find_combo;
 		Gtk.CheckButton search_all_notes;
 		Gtk.CheckButton case_sensitive;
 		Gtk.ScrolledWindow matches_window;
@@ -29,15 +29,15 @@ namespace Tomboy
 		Note current_note;
 		ArrayList current_matches;
 
-		static string [] previous_searches;
+		static ArrayList previous_searches;
 		static NoteFindDialog instance;
-		static Gdk.Pixbuf search_image;
-		static Gdk.Pixbuf stock_notes;
+		static Gdk.Pixbuf main_icon_large;
+		static Gdk.Pixbuf note_icon;
 
 		static NoteFindDialog ()
 		{
-			search_image = new Gdk.Pixbuf (null, "gnome-stock-searchtool.png");
-			stock_notes = GuiUtils.GetMiniIcon ("stock_notes.png");
+			main_icon_large = GuiUtils.GetIcon ("gnome-searchtool", 48);
+			note_icon = GuiUtils.GetIcon ("tomboy", 22);
 		}
 
 		public static NoteFindDialog GetInstance (Note note)
@@ -87,7 +87,7 @@ namespace Tomboy
 					Catalog.GetString ("Search All Notes") : 
 					Catalog.GetString ("Search Note"))
 		{
-			this.Icon = search_image;
+			this.IconName = "stock_search";
 
 			// For Escape (Close), Ctrl-G (Find next), and
 			// Ctrl-Shift-G (Find Previous)
@@ -97,17 +97,18 @@ namespace Tomboy
 			// Allow resizing if showing the results list
 			this.Resizable = search_all;
 
-			find_combo = new Gtk.Combo ();
-			find_combo.AllowEmpty = false;
-			find_combo.CaseSensitive = false;
-			find_combo.DisableActivate ();
+			find_combo = Gtk.ComboBoxEntry.NewText ();
+			find_combo.Changed += OnEntryChanged;
+			find_combo.Entry.ActivatesDefault = false;
 			find_combo.Entry.Activated += OnEntryActivated;
-			find_combo.Entry.Changed += OnEntryChanged;
-			if (previous_searches != null)
-				find_combo.PopdownStrings = previous_searches;
+			if (previous_searches != null) {
+				foreach (string prev in previous_searches) {
+					find_combo.AppendText (prev);
+				}
+			}
 
 			Gtk.Label label = new Gtk.Label (Catalog.GetString ("_Find:"));
-			label.MnemonicWidget = find_combo.Entry;
+			label.MnemonicWidget = find_combo;
 
 			search_all_notes = 
 				new Gtk.CheckButton (Catalog.GetString ("Search _All Notes"));
@@ -127,7 +128,7 @@ namespace Tomboy
 			widgets.ColumnSpacing = 4;
 			widgets.ShowAll ();
 
-			Gtk.Image image = new Gtk.Image (search_image);
+			Gtk.Image image = new Gtk.Image (main_icon_large);
 			image.Show ();
 
 			Gtk.HBox hbox = new Gtk.HBox (false, 2);
@@ -204,7 +205,6 @@ namespace Tomboy
 
 			this.Add (content_vbox);
 			this.DeleteEvent += HideOnDelete;
-			this.Shown += HighlightOnShown;
 		}
 
 		void MakeResultTreeView () 
@@ -232,9 +232,7 @@ namespace Tomboy
 
 			store = new Gtk.ListStore (types);
 			store.SetSortFunc (3 /* note */,
-					   new Gtk.TreeIterCompareFunc (CompareDates),
-					   IntPtr.Zero, 
-					   null);
+					   new Gtk.TreeIterCompareFunc (CompareDates));
 
 			tree = new Gtk.TreeView (store);
 			tree.HeadersVisible = true;
@@ -411,7 +409,7 @@ namespace Tomboy
 			if (current_matches == null || current_matches.Count == 0)
 				return;
 
-			Console.WriteLine ("Entering OnFindNextClicked...");
+			Logger.Log ("Entering OnFindNextClicked...");
 
 			for (int i = 0; i < current_matches.Count; i++) {
 				Match match = (Match) current_matches [i];
@@ -429,7 +427,7 @@ namespace Tomboy
 			// Else wrap to first match
 			JumpToMatch ((Match) current_matches [0]);
 
-			Console.WriteLine ("Leaving OnFindNextClicked...");
+			Logger.Log ("Leaving OnFindNextClicked...");
 		}
 
 		void OnFindPreviousClicked (object sender, EventArgs args)
@@ -468,15 +466,18 @@ namespace Tomboy
 			args.RetVal = true;
 		}
 
-		void HighlightOnShown (object sender, EventArgs args)
+		protected override void OnShown ()
 		{
+			find_combo.Entry.GrabFocus ();
+			base.OnShown ();
+
+			// Rehighlight any note text for the existing search.
 			if (current_matches != null)
 				HighlightMatches (current_matches, true);
 		}
 
 		void OnCaseSensitiveToggled (object sender, EventArgs args)
 		{
-			find_combo.CaseSensitive = case_sensitive.Active;
 			UpdateResults ();
 		}
 
@@ -577,39 +578,37 @@ namespace Tomboy
 		// results...
 		void EntryChangedTimeout (object sender, EventArgs args)
 		{
-			string text = find_combo.Entry.Text;
-			if (text == null || text == String.Empty)
+			if (SearchText == null)
 				return;
 
 			UpdateResults ();
-			AddToPreviousSearches (text);
+			AddToPreviousSearches (SearchText);
 		}
 
 		void AddToPreviousSearches (string text)
 		{
-			// Update previous searches, by either creating the
-			// initial list, or adding a new term to the list, or
-			// shuffling an existing term to the top...
-			if (previous_searches == null) {
-				previous_searches = new string [] { text };
+			// Update previous searches, by adding a new term to the
+			// list, or shuffling an existing term to the top...
+
+			if (previous_searches == null)
+				previous_searches = new ArrayList ();
+
+			bool repeat = false;
+
+			if (case_sensitive.Active) {
+				repeat = previous_searches.Contains (text);
 			} else {
-				int existing_idx = Array.IndexOf (previous_searches, text);
-				if (existing_idx > -1) {
-					// move the current search to the front of the list
-					string first = previous_searches [0];
-					previous_searches [0] = previous_searches [existing_idx];
-					previous_searches [existing_idx] = first;
-				} else {
-					string [] new_prev; 
-					new_prev = new string [previous_searches.Length + 1];
-					new_prev [0] = text;
-					// copy starting at index 1
-					previous_searches.CopyTo (new_prev, 1);
-					previous_searches = new_prev;
+				string lower = text.ToLower();
+				foreach (string prev in previous_searches) {
+					if (prev.ToLower() == lower)
+						repeat = true;
 				}
 			}
 
-			find_combo.PopdownStrings = previous_searches;
+			if (!repeat) {
+				previous_searches.Insert (0, text);
+				find_combo.PrependText (text);
+			}
 		}
 
 		void AddManagerChangeListeners ()
@@ -650,8 +649,8 @@ namespace Tomboy
 		{
 			CleanupMatches ();
 
-			string text = find_combo.Entry.Text;
-			if (text == null || text == String.Empty)
+			string text = SearchText;
+			if (text == null)
 				return;
 
 			if (!case_sensitive.Active)
@@ -710,7 +709,7 @@ namespace Tomboy
 				}
 			}
 			else if (current_note != null) {
-				Console.WriteLine ("Looking for {0}", text);
+				Logger.Log ("Looking for {0}", text);
 
 				current_matches = FindMatchesInBuffer (current_note.Buffer, 
 								       words, 
@@ -724,22 +723,9 @@ namespace Tomboy
 			}
 		}
 
-		[DllImport("libglib-2.0-0.dll")]
-                static extern IntPtr g_markup_escape_text (string text, int len);
-
-		// NOTE: Workaround a mapping bug in GLib.Markup.EscapeText,
-		// where the length passed to g_markup_escape_text is passed the
-		// character count, not the byte count of the string.
-		static string MarkupEscapeText (string source)
-		{
-			int len = System.Text.Encoding.UTF8.GetByteCount (source);
-			IntPtr markup = g_markup_escape_text (source, len);
-			return GLib.Marshaller.PtrToStringGFree (markup);
-		}
-
 		void AppendResultTreeView (Gtk.ListStore store, Note note, ArrayList matches)
 		{
-			string title = MarkupEscapeText (note.Title);
+			string title = GLib.Markup.EscapeText (note.Title);
 			string match_cnt;
 
 			match_cnt = 
@@ -749,7 +735,7 @@ namespace Tomboy
 					  matches.Count);
 
 			Gtk.TreeIter iter = store.Append ();
-			store.SetValue (iter, 0 /* icon */, stock_notes);
+			store.SetValue (iter, 0 /* icon */, note_icon);
 			store.SetValue (iter, 1 /* title */, title);
 			store.SetValue (iter, 2 /* match count */, match_cnt);
 			store.SetValue (iter, 3 /* note */, note);
@@ -766,7 +752,7 @@ namespace Tomboy
 
 		int CompareDates (Gtk.TreeModel model, Gtk.TreeIter a, Gtk.TreeIter b)
 		{
-			Console.WriteLine ("CompareDates Called!");
+			Logger.Log ("CompareDates Called!");
 
 			Note note_a = (Note) model.GetValue (a, 3 /* note */);
 			Note note_b = (Note) model.GetValue (b, 3 /* note */);
@@ -789,10 +775,15 @@ namespace Tomboy
 
 		public string SearchText
 		{
-			get { return find_combo.Entry.Text; }
+			get {
+				string text = find_combo.Entry.Text;
+				if (text == String.Empty)
+					return null;
+				return text;
+			}
 			set {
-				if (value != null)
-					find_combo.Entry.Text = value;
+				if (value != null && value != "")
+					find_combo.AppendText (value);
 			}
 		}
 	}
