@@ -2,7 +2,7 @@
 using System;
 using System.Collections;
 using System.Text;
-using Mono.Posix;
+using Mono.Unix;
 using System.Runtime.InteropServices;
 
 namespace Tomboy
@@ -12,23 +12,22 @@ namespace Tomboy
 		NoteManager manager;
 		Gtk.Tooltips tips;
 		Gtk.Image image;
+		PreferencesDialog prefs_dlg;
+		int icon_size_last = -1;
 
-		static Gdk.Pixbuf tintin;
-		static Gdk.Pixbuf tintin_large;
-		static Gdk.Pixbuf stock_notes;
+		static Gdk.Pixbuf note_icon;
 
 		static TomboyTray ()
 		{
-			tintin = GuiUtils.GetMiniIcon ("tintin.png");
-			tintin_large = GuiUtils.GetIcon ("tintin.png");
-			stock_notes = GuiUtils.GetMiniIcon ("stock_notes.png");
+			// Cache this since we use it a lot.
+			note_icon = GuiUtils.GetIcon ("tomboy", 16);
 		}
 
 		public TomboyTray (NoteManager manager) 
 			: base ()
 		{
 			this.manager = manager;
-			this.image = new Gtk.Image (tintin);
+			this.image = new Gtk.Image ();
 
 			this.CanFocus = true;
 			this.ButtonPressEvent += ButtonPress;
@@ -51,6 +50,8 @@ namespace Tomboy
 			tips.Sink ();
 
 			SetupDragAndDrop ();
+
+			InitSomeStuff ();
 		}
 
 		void ButtonPress (object sender, Gtk.ButtonPressEventArgs args) 
@@ -90,7 +91,7 @@ namespace Tomboy
 			Gtk.TextIter cursor = buffer.StartIter;
 			cursor.ForwardLines (1); // skip title
 
-			buffer.Insert (cursor, insert_text.ToString ());
+			buffer.Insert (ref cursor, insert_text.ToString ());
 
 			// Make the date string a small font...
 			cursor = buffer.StartIter;
@@ -134,7 +135,7 @@ namespace Tomboy
 		Gtk.Menu MakeRecentNotesMenu (Gtk.Widget parent) 
 		{
 			Gtk.Menu menu = new Gtk.Menu ();
-			menu.AttachToWidget (parent, null);
+			menu.AttachToWidget (parent, GuiUtils.DetachMenu);
 
 			bool enable_keybindings = (bool) 
 				Preferences.Get (Preferences.ENABLE_KEYBINDINGS);
@@ -156,7 +157,7 @@ namespace Tomboy
 			int max_size = 18;
 			int list_size = 0;
 
-			DateTime two_days_ago = DateTime.Now.AddDays (-2);
+			DateTime two_days_ago = DateTime.Today.AddDays (-2);
 
 			// List the i most recently changed notes, and any
 			// currently opened notes...
@@ -221,7 +222,7 @@ namespace Tomboy
 			display_name = FormatForLabel (display_name);
 
 			Gtk.ImageMenuItem item = new Gtk.ImageMenuItem (display_name);
-			item.Image = new Gtk.Image (stock_notes);
+			item.Image = new Gtk.Image (note_icon);
 			item.Data ["Note"] = note;
 			item.Activated += ShowNote;
 
@@ -288,11 +289,19 @@ namespace Tomboy
 			GuiUtils.PopupMenu (recent_menu, null);
 		}
 
+		void OnPreferencesResponse (object sender, Gtk.ResponseArgs args)
+		{
+			((Gtk.Widget) sender).Destroy ();
+			prefs_dlg = null;
+		}
+
 		public void ShowPreferences ()
 		{
-			PreferencesDialog prefs = new PreferencesDialog ();
-			prefs.Run ();
-			prefs.Destroy ();
+			if (prefs_dlg == null) {
+				prefs_dlg = new PreferencesDialog ();
+				prefs_dlg.Response += OnPreferencesResponse;
+			}
+			prefs_dlg.Present ();
 		}
 
 		public void ShowAbout ()
@@ -306,20 +315,25 @@ namespace Tomboy
 			};
 
 			string translators = Catalog.GetString ("translator-credits");
+			if (translators == "translator-credits")
+				translators = null;
 
-			Gnome.About about = 
-				new Gnome.About (
-					"Tomboy", 
-					Defines.VERSION,
-					Catalog.GetString ("Copyright \xa9 2004-2006 Alex Graveley"),
-					Catalog.GetString ("A simple and easy to use desktop " +
-							   "note-taking application."),
-					authors, 
-					documenters, 
-					translators,
-					tintin_large);
-			about.Icon = tintin_large;
-			about.Show ();
+			Gtk.AboutDialog about = new Gtk.AboutDialog ();
+			about.Name = "Tomboy";
+			about.Version = Defines.VERSION;
+			about.Logo = GuiUtils.GetIcon ("tomboy", 48);
+			about.Copyright = 
+				Catalog.GetString ("Copyright \xa9 2004-2006 Alex Graveley");
+			about.Comments = Catalog.GetString ("A simple and easy to use desktop " +
+							    "note-taking application.");
+			about.Website = "http://www.beatniksoftware.com/tomboy";
+			about.WebsiteLabel = Catalog.GetString("Homepage & Donations");
+			about.Authors = authors;
+			about.Documenters = documenters;
+			about.TranslatorCredits = translators;
+			about.IconName = "tomboy";
+			about.Run ();
+			about.Destroy ();
 		}
 
 		// Support dropping text/uri-lists and _NETSCAPE_URLs currently.
@@ -376,6 +390,57 @@ namespace Tomboy
 							insert_text.ToString ());
 			}
 		}
+
+		protected override void OnSizeAllocated (Gdk.Rectangle rect)
+		{
+			base.OnSizeAllocated (rect);
+
+			int icon_size = Math.Min (rect.Height, rect.Width);
+			if (icon_size_last != icon_size) {
+				icon_size_last = icon_size;
+				image.Pixbuf = GuiUtils.GetIcon (GetIconName (), icon_size);
+			}
+		}
+
+		void ReloadIcon ()
+		{
+			icon_size_last = -1;
+			QueueResize ();
+		}
+
+		// NOTHING TO SEE HERE
+		void InitSomeStuff ()
+		{
+			manager.NoteDeleted += OnNoteDeletedUpdateIcon;
+			manager.NoteAdded += OnNoteAddedUpdateIcon;
+			manager.NoteRenamed += OnNoteRenamedUpdateIcon;
+		}
+
+		void OnNoteDeletedUpdateIcon (object sender, Note changed)
+		{
+			if (changed.Title == "Tintin")
+				ReloadIcon ();
+		}
+
+		void OnNoteAddedUpdateIcon (object sender, Note changed)
+		{
+			if (changed.Title == "Tintin")
+				ReloadIcon ();
+		}
+
+		void OnNoteRenamedUpdateIcon (Note note, string old_title)
+		{
+			if (note.Title == "Tintin" || old_title == "Tintin")
+				ReloadIcon ();
+		}
+
+		string GetIconName ()
+		{
+			if (manager.Find ("Tintin") != null)
+				return "tintin";
+			return "tomboy";
+		}
+		// GO ABOUT YOUR BUSINESS
 	}
 
 	// 
