@@ -3,10 +3,12 @@ using System;
 using System.Collections;
 using System.Runtime.InteropServices;
 using System.Text;
-
 using System.IO;
 using System.Xml;
 
+using Mono.Unix;
+using Mono.Unix.Native;
+
 namespace Tomboy
 {
 	public class GuiUtils 
@@ -28,6 +30,13 @@ namespace Tomboy
 			push_in = true;
 		}
 
+		public static void DetachMenu (Gtk.Widget attach, Gtk.Menu menu)
+		{
+			// Do nothing.  Callers can use this to work around a
+			// Gtk#2 binding bug requiring a non-null detach
+			// delegate when calling Gtk.Menu.AttachToWidget.
+		}
+
 		static void DeactivateMenu (object sender, EventArgs args) 
 		{
 			Gtk.Menu menu = (Gtk.Menu) sender;
@@ -40,14 +49,13 @@ namespace Tomboy
 
 		// Place the menu underneath an arbitrary parent widget.  The
 		// parent widget must be set using menu.AttachToWidget before
-		// calling this
+		// calling this.
 		public static void PopupMenu (Gtk.Menu menu, Gdk.EventButton ev)
 		{
 			menu.Deactivated += DeactivateMenu;
 			menu.Popup (null, 
 				    null, 
 				    new Gtk.MenuPositionFunc (GetMenuPosition), 
-				    IntPtr.Zero, 
 				    (ev == null) ? 0 : ev.Button, 
 				    (ev == null) ? Gtk.Global.CurrentEventTime : ev.Time);
 
@@ -56,15 +64,23 @@ namespace Tomboy
 				menu.AttachWidget.State = Gtk.StateType.Selected;
 		}
 
-		public static Gdk.Pixbuf GetIcon (string resource_name) 
+		public static Gdk.Pixbuf GetIcon (string resource_name, int size) 
 		{
-			return new Gdk.Pixbuf (null, resource_name);
-		}
+			try {
+				return Gtk.IconTheme.Default.LoadIcon (resource_name, size, 0);
+			} catch (GLib.GException e) {
+				Logger.Log ("Unable to load icon '{0}': {1}", 
+					    resource_name, 
+					    e.Message);
+			}
 
-		public static Gdk.Pixbuf GetMiniIcon (string resource_name) 
-		{
-			Gdk.Pixbuf noicon = new Gdk.Pixbuf (null, resource_name);
-			return noicon.ScaleSimple (24, 24, Gdk.InterpType.Nearest);
+			try {
+				Gdk.Pixbuf ret = new Gdk.Pixbuf (null, resource_name + ".png");
+				return ret.ScaleSimple (size, size, Gdk.InterpType.Bilinear);
+			} catch (ArgumentException e) {
+			}
+
+			return null;
 		}
 
 		public static Gtk.Button MakeImageButton (Gtk.Image image, string label)
@@ -280,7 +296,7 @@ namespace Tomboy
 				if (s.StartsWith ("file:////"))
 					s = s.Replace ("file:////", "file:///");
 
-				Console.WriteLine ("uri = {0}", s);
+				Logger.Log ("uri = {0}", s);
 				try {
 					Uri uri = new Uri (s);
 					if (uri != null)
@@ -435,7 +451,9 @@ namespace Tomboy
 
 		public void Erase ()
 		{
-			buffer.Delete (Start, End);
+			Gtk.TextIter start_iter = Start;
+			Gtk.TextIter end_iter = End;
+			buffer.Delete (ref start_iter, ref end_iter);
 		}
 
 		public void Destroy ()
@@ -621,7 +639,7 @@ namespace Tomboy
 			this.is_important = false;
 
 			this.menu = menu;
-			this.menu.AttachToWidget (this, null);
+			this.menu.AttachToWidget (this, GuiUtils.DetachMenu);
 			this.menu.Deactivated += ReleaseButton;
 
 			this.image = image;
@@ -725,4 +743,127 @@ namespace Tomboy
 			ShowForToolbarStyle (args.Style);
 		}
 	}
+
+	public class Application
+	{
+		static Gnome.Program program;
+
+		public static void Initialize (string locale_dir, 
+					       string display_name, 
+					       string process_name,  
+					       string [] args)
+		{
+			SetProcessName (process_name);
+
+			// Initialize GETTEXT
+			Catalog.Init (locale_dir, Defines.GNOME_LOCALE_DIR);
+
+			Gtk.Application.Init ();
+			program = new Gnome.Program ("Tomboy", 
+						     Defines.VERSION, 
+						     Gnome.Modules.UI, 
+						     args);
+
+			RegisterSignalHandlers ();
+		}
+
+		[DllImport("libc")]
+		private static extern int prctl (int option, 
+						 byte [] arg2, 
+						 IntPtr arg3, 
+						 IntPtr arg4, 
+						 IntPtr arg5);
+
+		// From Banshee: Banshee.Base/Utilities.cs
+		public static void SetProcessName (string name)
+		{
+			if (prctl (15 /* PR_SET_NAME */, 
+				   Encoding.ASCII.GetBytes (name + "\0"),
+				   IntPtr.Zero, 
+				   IntPtr.Zero, 
+				   IntPtr.Zero) != 0)
+				throw new ApplicationException (
+					"Error setting process name: " +
+					Mono.Unix.Native.Stdlib.GetLastError ());
+		}
+
+		static void RegisterSignalHandlers ()
+		{
+			// Connect to SIGTERM and SIGINT, so we don't lose
+			// unsaved notes on exit...
+			Stdlib.signal (Signum.SIGTERM, OnExitSignal);
+			Stdlib.signal (Signum.SIGINT, OnExitSignal);
+		}
+
+		public static void RegisterSessionManagerRestart (string executable_path,
+								  string[] args, 
+								  string[] environment)
+		{
+			if (executable_path == null)
+				return;
+
+			// Restart if we are running when the session ends or at crash...
+			Gnome.Client client = Gnome.Global.MasterClient ();
+			client.RestartStyle = 
+				Gnome.RestartStyle.IfRunning | Gnome.RestartStyle.Immediately;
+			client.Die += OnSessionManagerDie;
+
+			foreach (string env in environment) {
+				string [] split = env.Split (new char [] { '=' }, 2);
+				if (split.Length == 2) {
+					client.SetEnvironment (split[0], split[1]);
+				}
+			}
+
+			// Get the args for session restart... 	 
+			string [] restart_args = new string [args.Length + 1]; 	 
+			restart_args [0] = executable_path;
+			args.CopyTo (restart_args, 1);
+			client.SetRestartCommand (restart_args.Length, restart_args);
+		}
+
+		static void OnSessionManagerDie (object sender, EventArgs args)
+		{
+			// Don't let the exit signal run, which would cancel
+			// session management.
+			QuitMainLoop ();
+		}
+
+		static void CancelSessionManagerRestart ()
+		{
+			Gnome.Client client = Gnome.Global.MasterClient ();
+			client.RestartStyle = Gnome.RestartStyle.IfRunning;
+			client.Flush ();
+		}
+
+		static void OnExitSignal (int signal)
+		{
+			// Don't auto-restart after exit/kill.
+			CancelSessionManagerRestart ();
+
+			if (ExitingEvent != null)
+				ExitingEvent (null, new EventArgs ());
+
+			if (signal >= 0)
+				System.Environment.Exit (0);
+		}
+
+		public static event EventHandler ExitingEvent;
+
+		public static void Exit (int exitcode)
+		{
+			OnExitSignal (-1);
+			System.Environment.Exit (exitcode);
+		}
+
+		public static void StartMainLoop ()
+		{
+			program.Run ();
+		}
+
+		public static void QuitMainLoop ()
+		{
+			Gtk.Main.Quit ();
+		}
+	}
 }
