@@ -3,10 +3,12 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkwindow.h>
 #include <gdk/gdkx.h>
+#include <gtk/gtk.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
 #include "config.h"
+#include "tomboykeybinder.h"
 #include "tomboyutil.h"
 
 /* Uncomment the next line to print a debug trace. */
@@ -18,19 +20,6 @@
 #  define TRACE(x) do {} while (FALSE);
 #endif
 
-void
-tomboy_widget_set_bg_pixmap (GtkWidget *applet, GdkPixmap *pixmap)
-{
-	GtkStyle *style;
-
-	style = gtk_style_copy (GTK_WIDGET (applet)->style);
-	if (style->bg_pixmap[GTK_STATE_NORMAL])
-		g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
-	style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
-	gtk_widget_set_style (GTK_WIDGET (applet), style);
-	g_object_unref (style);
-}
-
 gint
 tomboy_window_get_workspace (GtkWindow *window)
 {
